/* ************************************************************************* *
 *                                                                           *
 *    clnt_lsize.c,v
 *    Client procedure to change the size of a PFSLib file
 *                                                                           *
 *    Copyright (C) 1995 A. Bode, S. Lamberts, T. Ludwig, C. R"oder          *
 *                                                                           *
 *    PFSLib (Parallel I/O on workstations)                                  *
 *                                                                           *
 *    PFSLib offers parallel access to files for a parallel application      *
 *    running on a cluster of workstations.                                  *
 *    It is intended but not restricted to be used in message passing        *
 *    applications based on PVM, NXLib, MPI, and other.                      *
 *                                                                           *
 *    PFSLib consists of a LIBRARY, deamon PROGRAMS, and utility PROGRAMS.   *
 *                                                                           *
 *    PFSLib is free software; you can redistribute the LIBRARY and/or       *
 *    modify it under the terms of the GNU Library General Public            *
 *    License as published by the Free Software Foundation; either           *
 *    version 2 of the License, or (at your option) any later version.       *
 *    You can redistribute the daemon PROGRAMS and utility PROGRAMS          *
 *    and/or modify them under the terms of the GNU General Public           *
 *    License as published by the Free Software Foundation; either           *
 *    version 2 of the License, or (at your option) any later version.       *
 *                                                                           *
 *    PFSLib is distributed in the hope that it will be useful,              *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *    Library General Public License and GNU General Public License          *
 *    for more details.                                                      *
 *                                                                           *
 *    You should have received a copy of the GNU Library General Public      *
 *    License and the GNU General Public License along with this             *
 *    library; if not, write to the Free                                     *
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     *
 *                                                                           *
 *    Contact to the authors:                                                *
 *                                                                           *
 *    electronic mail: pfslib@informatik.tu-muenchen.de                      *
 *                                                                           *
 *    paper mail:      Prof. Dr. A. Bode                                     *
 *                     Lehrstuhl f"ur Rechnertechnik und                     *
 *                     Rechnerorganisation                                   *
 *                     Institut f"ur Informatik                              *
 *                     Technische Universit"at M"unchen                      *
 *                     80290 M"unchen                                        *
 *                     Germany                                               *
 *                                                                           *
 *    This project was partially funded by a research grant form Intel       *
 *    Corporation.                                                           *
 *                                                                           *
 * ************************************************************************* */


/* ************************************************************************* *
 *                                                                           *
 *  RCS Filename : clnt_lsize.c,v
 *  RCS Date     : 1996/05/29 15:05:13
 *  RCS Revision : 1.3
 *  RCS Author   : lamberts
 *  RCS State    : V2_0_B
 *                                                                           *
 *  Authors: Stefan Lamberts, Christian R"oder                               *
 *                                                                           *
 * ************************************************************************* */
#ifndef lint
static void *rcs_id = "clnt_lsize.c,v 1.3 1996/05/29 15:05:13 lamberts V2_0_B";
#endif

/* ************************************************************************* *
 * Include files                                                             *
 * ************************************************************************* */

#include <rpc/rpc.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "pfsd.h"
#include "clnt_defines.h"
#include "clnt_macros.h"
#include "pfslib_errno.h"

/* ************************************************************************* *
 * External declarations                                                     *
 * ************************************************************************* */

extern int          _pfslib_maxfiles;
extern CLIENT      *_pfslib_clnt;
extern pfslib_fhdl *_pfslib_ftbl;

#ifdef ANSI_C
#define _PH(a) a
#else  /* ANSI_C */
#define _PH(a) ()
#endif /* ANSI_C */

extern void pfslib_perror _PH((char *str));

#undef _PH

/* ************************************************************************* *
 * Procedures                                                                *
 * ************************************************************************* */

/* ************************************************************************* *
 * Increase the size of a PFSLib file.                                       *
 * ************************************************************************* */
long _lsize
#ifdef ANSI_C
(int fd, off_t offset, int whence)
#else  /* ANSI_C */
(fd, offset, whence)
int fd;
off_t offset;
int whence;
#endif /* ANSI_C */
{
  lsizeres    *result;
  fpmoveargs   arg;
  long size;

  fd -= VFDOFFSET;

  /* Check arguments */
  if ((fd < 0) || (fd >= _pfslib_maxfiles) ||
      (_pfslib_ftbl[fd].vfd == -1))
  {
    errno = EPFSLBADF;
    return (-1);
  }
  
  if ((whence != SIZE_SET) &&
      (whence != SIZE_CUR) &&
      (whence != SIZE_END))
  {
    errno = EPFSLINVAL;
    return (-1);
  }
  
  arg.fh     = _pfslib_ftbl[fd];
  arg.whence = whence;
  arg.offset = offset;


  if ((result = pfsd_lsize_1(&arg, _pfslib_clnt)) == NULL)
  {
    clnt_perror(_pfslib_clnt, "_lsize(): pfsd_lsize_1()");
    return (-1);
  }

  ERR_RESULT(lsizeres_u, xdr_lsizeres);
  
  size = (long)result->lsizeres_u.size;

  xdr_free(xdr_lsizeres, (char *)result);
  
  return (size);
}

/* ************************************************************************* *
 * Increase the size of a PFSLib file. Exit on failure.                      *
 * ************************************************************************* */
long lsize
#ifdef ANSI_C
(int filedes, off_t offset, int whence)
#else  /* ANSI_C */
(filedes, offset, whence)
int filedes;
off_t offset;
int whence;
#endif /* ANSI_C */
{
  long res;

  if ((res = _lsize( filedes, offset, whence)) < 0)
  {
    pfslib_perror("lsize()");
    exit (1);
  }

  return(res);
}
